angular.module('starter.controllers')
.controller('MoodEditCtrl', function ($scope, $q, $state, $timeout, $ionicLoading, $ionicModal, $ionicPopup, $ionicHistory, $storage, moodService, photoService, $rootScope, imagePreviewService) {
	var origin;
	$scope.$on("$state.call", function (event, params) {
		load(params.mood);
		$scope.option = { useOriginalImage: false, private: false };
		getDraft();
		var on$stateChangeStart = $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
			if (!toState.name.startsWith("app.mood-edit")) {
				confirm().then(function () {
					on$stateChangeStart();
					$ionicHistory.goBack();
				});
				event.preventDefault();
			}
		});
	});
	$scope.getDraft = getDraft;
	function getDraft() {
		($scope.draftPromise = moodService.getDraft()).then(function (response) {
			$scope.draft = response.data.content;
		});
	}
	$scope.$on("mood.create", getDraft);
	$scope.$on("mood.modify", getDraft);
	$scope.$on("mood.delete", getDraft);
	$scope.$on("mood.delete", function (event, mood) {
		if (mood != $scope.mood.id) return;
		load({ rich: true, title: '', richContent: [] });
	});
	$scope.submit = function () {
		if (!$scope.mood.title) {
			$ionicPopup.alert({ template: "请填写标题" });
			return;
		}
		if (!pictureCount()) {
			$ionicPopup.alert({ template: "请至少包含一张图片" });
			return;
		}
		$ionicLoading.show({ template: "<ion-spinner/>" });
		$scope.mood.isDraft = false;
		moodService.post($scope.mood, $scope.option).then(function () {
			$ionicLoading.hide();
			save();
			$ionicModal.fromTemplateUrl('templates/share-success.html', {
				scope: $scope,
				animation: 'slide-in-up'
			}).then(function (modal) {
				return modal.wait();
			}).then($state.resolve);
			if ($scope.mood.id) {
				$ionicHistory.clearCache(['app.mood_id=' + $scope.mood.id]);
				$storage.remove('mood', $scope.mood.id);
			}
		}, function () {
			$ionicLoading.show({ template: "创建失败" });
		});
	};
	$scope.save = function () {
		$ionicLoading.show({ template: "<ion-spinner/>" });
		$scope.mood.isDraft = true;
		moodService.post($scope.mood, $scope.option).then(function () {
			save();
			$ionicLoading.show({ template: "已保存至草稿箱", duration: 2000 });
		}, function () {
			$ionicLoading.show({ template: "保存失败" });
		});
	};
	$scope.insert = function (index, type) {
		switch (type) {
			case 'text':
				$scope.mood.richContent.splice(index, 0, { type: 'text', value: '' });
				break;
			case 'picture':
				var c = pictureCount();
				if (c >= 9) {
					$ionicPopup.alert({ template: "最多只能添加9张照片" });
					return;
				}
				photoService.getPhoto(true, 9 - c).then(function (url) {
					if (typeof (url) == 'string')
						url = [url];
					$q.all(url.map(function (url) {
						return photoService.ImageInfo(url).then(function (info) {
							if (info.exif.Orientation && info.exif.Orientation != 1) {
								return photoService.rotateImg(info)
									.then(function (imgUrl) {
										return {type: 'picture', value: imgUrl};
									})
							} else {
								return {type: 'picture', value: url};
							}
						});
					})).then(function (result) {
						[].splice.apply(
							$scope.mood.richContent,
							[index, 0].concat(result)
						);
					});
				});
				break;
		}
	};
	$scope.remove = function (index) {
		$scope.mood.richContent.splice(index, 1);
	};
	$scope.load = function (mood) {
		return confirm().then(function () {
			load(mood);
		});
	};
	$scope.open = function (mood) {
		return this.load(mood).then(function () {
			$state.go("^.content");
		});
	};
	$scope.imagePreview = function (component) {
		var components = pictures();
		var urls = components.map(function (component) {
			return component.value;
		});
		var index = components.indexOf(component);
		imagePreviewService.show(urls, index);
	};
	function pictures() {
		return $scope.mood.richContent.filter(function (component) {
			return component.type == 'picture';
		});
	}
	function confirm() {
		if (moodService.equals(origin, $scope.mood))
			return $q.resolve();
		return $ionicPopup.confirm({
			template: "放弃当前的修改？"
		}).then(function (response) {
			return response ? $q.resolve() : $q.reject();
		});
	}
	function load(mood) {
		origin = mood;
		$scope.mood = moodService.copy(mood);
	}
	function save() {
		origin.title = $scope.mood.title;
		origin.richContent = angular.copy($scope.mood.richContent);
	}
	function pictureCount(){
		return $scope.mood.richContent.filter(function (component) {
			return component.type == 'picture';
		}).length;
	}
});
