angular.module('starter.controllers')
	.controller('translateTextCtrl', function ($scope, $ionicModal, $ionicPopup, $ionicLoading, $interval, translateService, $speak, $record, $cordovaFile) {
		$scope.language = translateService.language;
		$scope.translation = {
			source: '',
			destination: ''
		};
		$scope.onkeypress = function ($event) {
			if ($event.keyCode == 13) {
				$event.preventDefault();
				if (!$scope.translation.source) return;
				$event.target.blur();
				$ionicLoading.show({template: "<ion-spinner/><div>正在翻译</div>"});
				translateService.translate($scope.translation.source, ($scope.audioDestinationLanguage || translateService.language.destination).code)
					.then(function (result) {
						$ionicLoading.hide();
						var id = +new Date();
						$scope.translation.destination = result.translatedText;
						var translation = {
							source: {
								language: translateService.language.source,
								text: $scope.translation.source
							},
							destination: {
								language: $scope.audioDestinationLanguage || translateService.language.destination,
								text: result.translatedText
							},
							id: id
						};
						translateService.history.push(translation);
						if (translateService.history.length > 20) {
							var earlist = translateService.history.shift();
							translateService.db.remove('history', earlist.id);
						}
						translateService.db.put('history', translation.id, translation);
					}, function () {
						$ionicLoading.show({template: "请求失败"});
					});
			}
		};
		$scope.translateAudio = function (audio, language) {
			$ionicLoading.show({template: "<ion-spinner/><div>正在翻译</div>"});
			var source = language;
			var destination = language == $scope.language.source ? $scope.language.destination : $scope.language.source;
			$scope.audioDestinationLanguage = destination;
			translateService.translateAudio(audio, source.code, destination.code)
				.then(function (result) {
					$ionicLoading.hide();
					var id = +new Date();
					$scope.translation.source = result.originalText;
					$scope.translation.destination = result.translatedText;
					setTimeout(function () {
						$scope.speakButton.trigger('click');
					}, 0);
					var translation = {
						source: {
							language: source,
							text: $scope.translation.source
						},
						destination: {
							language: destination,
							text: result.translatedText
						},
						id: id
					};
					translateService.history.push(translation);
					if (translateService.history.length > 20) {
						var earlist = translateService.history.shift();
						translateService.db.remove('history', earlist.id);
					}
					translateService.db.put('history', translation.id, translation);
				}, function (e) {
					$ionicLoading.hide();
					if (e == "has no result") {
						$ionicPopup.alert({template: "声音没有识别成功，可能是声音太轻。"});
						return;
					}
					$ionicLoading.show({template: "请求失败"});
				});
		};
		$scope.audioDestinationLanguage;
		$scope.ontouch = function (language) {
			$scope.recording = {language: language};
		};
		$scope.onrelease = function (language) {
			$scope.recording = undefined;
		};
		var i, j;
		$scope.$watch('recording', function (value, oldValue) {
			if (value && !oldValue) {
				$scope.recording.time = 0;
				i = $interval(function () {
					$scope.recording.time++;
				}, 1000);
			}
		});
		$scope.$watch("recording.time", function (value) {
			if (value == 60)
				$scope.recording = undefined;
		});
		$scope.$watch('recording', function (value) {
			if (!value)
				$interval.cancel(i);
		});
		$scope.$watch('recording', function (value, oldValue) {
			if (value && !oldValue) {
				j = $interval(function () {
					window.plugins.audioRecorderAPI.getMaxAmplitude(function (amplitude) {
						$scope.recording.amplitude = amplitude;
					});
				}, 100);
			}
		});
		$scope.$watch('recording', function (value) {
			if (!value)
				$interval.cancel(j);
		});
		$scope.$watch('recording', function (value, oldValue) {
			if (value && !oldValue)
				$record.start();
		});
		$scope.$watch('recording', function (value, oldValue) {
			if (!value && oldValue)
				$record.stop()
					.then(function (url) {
						var name = url.substr(url.lastIndexOf('/') + 1);
						$cordovaFile.readAsDataURL(cordova.file.dataDirectory, name)
							.then(function (url) {
								var base64 = url.substr(url.indexOf("base64,") + "base64,".length);
								$scope.translateAudio(base64, oldValue.language);
							});
					}, function (e) {
						console.log(e);
					});
		});
		$scope.$on("$ionicView.leave", function () {
			$speak.stop();
		});
	});
