angular.module('starter.directives')
.directive('paging', function ($rest) {
	var pageSize = 20;
	return {
		restrict: 'A',
		scope: {
			ngModel: '='
		},
		controller: function ($scope, $attrs) {
			var $this = this;
			this.page = function (direction) {
				this.paging = direction;
				var number = $scope.ngModel[$scope.field].number;
				direction == 'up' ? number-- : number++;
				$rest.get(
					$scope.type, $scope.id, $scope.relation,
					$scope.subtype,
					number, $attrs.pageSize || pageSize
				).then(function (response) {
					$scope.ngModel[$scope.field] = response.data;
					$this.paging = null;
				});
			};
			this.scope = $scope;
			this.paging;
		},
		compile: function (tElement, tAttrs, tTransclude) {
			tElement.prepend($("<pager direction=up></pager>"));
			tElement.append($("<pager direction=down></pager>"));
			tElement.prepend($("<div name=spinner style=display:flex;justify-content:center;><ion-spinner></ion-spinner></div>").addClass('ng-hide'));
			tElement.prepend($("<div name=empty style=display:flex;justify-content:center;color:lightgray;>没有数据可以显示</div>").addClass('ng-hide'));
			tElement.prepend($("<div name=error style=display:flex;justify-content:center;color:lightgray;>请求错误</div>").addClass('ng-hide'));

			return function (scope, element, attrs) {
				attrs.$observe('paging', function (value) {
					if (!value)
						return;
					var a = attrs.paging.split('/');
					if (a.length >= 4) {
						var type = a[1];
						var id = a[2];
						var relation = attrs.paging.substr(type.length + id.length + 3);
					} else {
						var type, id;
						var relation = a[1];
					}
					var field = attrs.ngField;
					var subtype = attrs.ngType || $rest.type[type][relation];
					var spinner = $(element[0]).children('*[name="spinner"]');
					spinner.removeClass('ng-hide');
					var empty = $(element[0]).children('*[name="empty"]');
					empty.addClass('ng-hide');
					var error = $(element[0]).children('*[name="error"]');
					error.addClass('ng-hide');
					$rest.get(type, id, relation, subtype, 0, attrs.pageSize || pageSize).then(function (response) {
						spinner.addClass('ng-hide');
						scope.ngModel[field] = response.data;
						if (!response.data.totalPages)
							empty.removeClass('ng-hide');
					}, function (e) {
						spinner.addClass('ng-hide');
						error.removeClass('ng-hide');
					});
					scope.type = type;
					scope.id = id;
					scope.relation = relation;
					scope.field = field;
					scope.subtype = subtype;
				});
				//workaround: 处理往空的列表里添加东西的情况，比如用户添加评论
				scope.$watch(function () {
					return scope.field in scope.ngModel ?
						scope.ngModel[scope.field].content.length :
						NaN;
				}, function (value) {
					if (!value) return;
					$(element[0]).children('*[name="empty"]').addClass('ng-hide');
				});
			};
		}
	}
})
.directive('pager', function () {
	return {
		require: '^paging',
		restrict: 'E',
		scope: {
			direction: '@'
		},
		templateUrl: "templates/pager.html",
		replace: true,
		link: function (scope, element, attrs, paging) {
			scope.paging = paging;
			scope.page = function () {
				paging.page(scope.direction);
			};
			scope.isFirst = function () {
				return paging.scope.ngModel[paging.scope.field].number == 0;
			};
			scope.isLast = function () {
				var page = paging.scope.ngModel[paging.scope.field];
				return page.number + 1 >= page.totalPages;
			};
			scope.comment = function () {
				var ngModel = paging.scope.ngModel;
				return ngModel ? ngModel[paging.scope.field] : undefined;
			};
		}
	}
});