function YbbLayer(map, poiService) {
	var injector = angular.element(document.body).injector();
	var $q = injector.get('$q');
	var m = maps.m;
	var pois = {};
	var add = function (poi) {
		pois[poi.id] = poi;
		var marker = new m.YbbMarker(poi);
		poi.marker = marker;
		map.addOverlay(marker);
		marker.addEventListener('click', onclick);
		function onclick(e) {
			if (marker.element.badge.style.display != 'none') {
				setTimeout(function () {
					if (map.getZoom() >= 18) {
						var window = InfoWindow(marker);
						var location = { lat: marker.poi.location.lat, lng: marker.poi.location.lon };
						if (maps.m == maps.gmap)
							location = eviltransform.wgs2gcj(location.lat, location.lng);
						else
							location = eviltransform.wgs2bd(location.lat, location.lng);
						map.openInfoWindow(new m.InfoWindow(window), m.Location(location));
					} else {
						map.addEventListener(maps.m == maps.gmap ? 'bounds_changed' : 'moveend', function onmoveend() {
							map.zoomIn();
							map.removeEventListener(maps.m == maps.gmap ? 'bounds_changed' : 'moveend', onmoveend);
						})
						var latlng = { lat: poi.location.lat, lng: poi.location.lon };
						if (maps.m == maps.gmap)
							latlng = eviltransform.wgs2gcj(latlng.lat, latlng.lng);
						else
							latlng = eviltransform.wgs2bd(latlng.lat, latlng.lng);
						map.panTo(m.Location(latlng));
					}
				}, 0);
			} else
				$this.onclick(poi);
		}
		//百度地图自定义覆盖物在手机上不响应click事件，先用touchstart代替
		marker.addEventListener('touchstart', onclick);
	};
	if (maps.m == maps.gmap)
		map.addEventListener('bounds_changed', refresh);
	else {
		map.addEventListener('zoomend', refresh);
		map.addEventListener('moveend', refresh);
	}
	var tag = {};
	function filter() {
		for (var i in pois) {
			var poi = pois[i];
			poi.marker.show();
			poi.marker.setBadge(null);
		}
		var dbscan = new DBSCAN();
		var locations = [];
		for (var i in pois) {
			var poi = pois[i];
			if (!$this.tag[poi.type]) {
				poi.marker.hide();
				continue;
			}
			var location = m.Location({ lat: poi.location.lat, lng: poi.location.lon });
			var pixel = maps.m == maps.gmap ?
				poi.marker.getProjection().fromLatLngToDivPixel(location) :
				map.pointToOverlayPixel(location);
			var location = [pixel.x, pixel.y];
			location.poi = poi;
			locations.push(location);
		};
		var clusters = dbscan.run(locations, 72, map.getZoom() > 8 ? 2 : 1);
		for (var i in clusters) {
			var cluster = clusters[i];
			var zoom = map.getZoom();
			var marker = locations[cluster[0]].poi.marker;
			marker.setBadge(zoom > 8 ? cluster.length : '+');
			marker.cluster =
				cluster.map(function (i) {
					return locations[i].poi;
				});
			cluster.shift();
			for (var j in cluster)
				locations[cluster[j]].poi.marker.hide();
		}
	}
	function refresh() {
		var center = map.getCenter();
		var southwest = map.getBounds().getSouthWest();
		var radius = haversine(maps.normalize(center), maps.normalize(southwest));
		var zoom = map.getZoom();
		poiService.search(maps.normalize(center), radius, zoom > 8 ? 18 : zoom).then(function (response) {
			//假设refresh只在enabled状态被调用，无法保证这期间用户没有disable
			if (disabled) return;
			var data = response.data;
			var generation = data.filter(function (poi) {
				return !(poi.id in pois);
			});
			generation.forEach(add);
			if (maps.m == maps.gmap)
				$q.all(generation.map(function (poi) {
					return $q(function (resolve, reject) {
						poi.marker.addEventListener('add', function () {
							poi.marker.hide();
							resolve();
						});
					});
				})).then(filter);
			else
				filter();
		}, $this.onerror);
		filter();
	}
	function disable() {
		for (var i in pois) {
			var poi = pois[i];
			poi.marker.hide();
		}
		if (maps.m == maps.gmap)
			map.removeEventListener('bounds_changed', refresh);
		else {
			map.removeEventListener('zoomend', refresh);
			map.removeEventListener('moveend', refresh);
		}
		disabled = true;
	}
	function enable() {
		for (var i in pois) {
			var poi = pois[i];
			poi.marker.show();
		}
		refresh();
		if (maps.m == maps.gmap)
			map.addEventListener('bounds_changed', refresh);
		else {
			map.addEventListener('zoomend', refresh);
			map.addEventListener('moveend', refresh);
		}
		disabled = false;
	}
	this.onerror = function (e) { };
	this.onclick = function (poi) { };

	var disabled = false;
	var $this = {
		filter: filter,
		refresh: refresh,
		disable: disable,
		enable: enable,
		tag: tag
	};
	return $this;
	function InfoWindow(marker) {
		var div = document.createElement('div');
		div.className = "ybb-layer-popup";
		$(div).append(marker.cluster.map(Item));
		return div;
		function Item(poi) {
			var a = document.createElement('a');
			a.ontouchstart = function () {
				$this.onclick(poi);
			};
			var img = document.createElement('img');
			img.src = environment.imagePrefix + poi.picture;
			var span = document.createElement('span');
			span.innerHTML = poi.name;
			a.appendChild(img);
			a.appendChild(span);
			return a;
		}
	}
}
