maps = {};
maps.Google = function () {
	google.maps.Map.prototype.zoomIn = function () {
		this.setZoom(this.getZoom() + 1);
	};
	google.maps.Map.prototype.zoomOut = function () {
		this.setZoom(this.getZoom() - 1);
	};
	google.maps.Map.prototype.centerAndZoom = function (center, zoom) {
		this.setCenter(center);
		this.setZoom(zoom);
	};
	google.maps.Map.prototype.addOverlay = function (overlay) {
		overlay.setMap(this);
	};
	google.maps.Map.prototype.removeOverlay = function (overlay) {
		overlay.setMap(null);
	};
	google.maps.Map.prototype.addEventListener = function (name, f) {
		if (!f.listener) f.listener = {};
		f.listener[name] = google.maps.event.addListener(this, name, f);
	};
	google.maps.Map.prototype.removeEventListener = function (name, f) {
		f.listener[name].remove();
	};
	google.maps.Map.prototype.openInfoWindow = function (window, position) {
		window.setPosition(position);
		window.open(this);
	};
	google.maps.Map.prototype.panToLocations = function (latlngs) {
		if (!latlngs.length) return;
		var bounds = new google.maps.LatLngBounds();
		for (var i in latlngs)
			bounds.extend(latlngs[i]);
		this.fitBounds(bounds);
	};
	function YbbMarker(poi) {
		this.poi = poi;
	}
	YbbMarker.prototype = new google.maps.OverlayView();
	YbbMarker.prototype.onAdd = function () {
		var element = new maps.Element(this.poi);
		this.getPanes().overlayMouseTarget.appendChild(element.div);
		var $this = this;
		google.maps.event.addDomListener(element.div, 'click', function () {
			google.maps.event.trigger($this, 'click');
		});
		this.element = element;
		google.maps.event.trigger(this, 'add');
		return element.div;
	}
	YbbMarker.prototype.draw = function () {
		var wgs = this.poi.location;
		var gcj = eviltransform.wgs2gcj(wgs.lat, wgs.lon);
		var pixel = this.getProjection().fromLatLngToDivPixel(new google.maps.LatLng(gcj.lat, gcj.lng));
		var div = this.element.div;
		div.style.left = pixel.x + "px";
		div.style.top = pixel.y + "px";
	}
	YbbMarker.prototype.addEventListener = function (event, f) {
		google.maps.event.addListener(this, event, f);
	}
	YbbMarker.prototype.setBadge = function (value) {
		this.element.setBadge(value);
	}
	YbbMarker.prototype.setLabel = function (value) {
		this.element.setLabel(value);
	}
	YbbMarker.prototype.show = function () {
		this.element.div.style.display = '';
	}
	YbbMarker.prototype.hide = function () {
		this.element.div.style.display = 'none';
	}
	function PositionIndicator(position) {
		this.position = position;
	}
	PositionIndicator.prototype = new google.maps.OverlayView();
	PositionIndicator.prototype.onAdd = function () {
		var element = new maps.PositionIndicator.Element(this.position);

		this.getPanes().overlayMouseTarget.appendChild(element.img);
		this.getMap().addOverlay(element.circle);
		this.element = element;

		return element.img;
	}
	PositionIndicator.prototype.draw = function () {
		var gcj = eviltransform.wgs2gcj(this.position.latitude, this.position.longitude);
		var pixel = this.getProjection().fromLatLngToDivPixel(new google.maps.LatLng(gcj.lat, gcj.lng));
		var img = this.element.img;
		img.style.left = pixel.x + "px";
		img.style.top = pixel.y + "px";
	}
	PositionIndicator.prototype.addEventListener = function (event, f) {
		this.element.img['on' + event] = f;
	}
	PositionIndicator.prototype.setHeading = function (rotation) {
		this.element.setHeading(rotation);
	}
	PositionIndicator.prototype.setPosition = function (position) {
		this.position = position;
		this.draw();
		var gcj = eviltransform.wgs2gcj(position.latitude, position.longitude);
		this.element.circle.setCenter(new google.maps.LatLng(gcj.lat, gcj.lng));
		this.element.circle.setRadius(Math.min(position.accuracy || Infinity, 2000));
	}
	PositionIndicator.prototype.setHeading = function (setHeading) {
		return function (heading) {
			if (!this.element) return;
			setHeading.call(this, heading);
		};
	}(PositionIndicator.prototype.setHeading)
	PositionIndicator.prototype.setPosition = function (setPosition) {
		return function (position) {
			if (!this.element) return;
			setPosition.call(this, position);
		};
	}(PositionIndicator.prototype.setPosition)
	return {
		Map: function (element) {
			var map = new google.maps.Map(element, {
				disableDefaultUI: true,
				styles: [{
					featureType: "poi",
					stylers: [{
						visibility: "off"
					}]
				}]
			});
			return map;
		},
		Marker: function (options) {
			var marker = new google.maps.Marker({
				position: options.position,
				icon: Icon(.5),
				zIndex: 0,
				map: options.map
			});
			marker.activate = function () {
				this.setIcon(Icon(2 / 3));
				this.setZIndex(1);
			};
			marker.deactivate = function () {
				this.setIcon(Icon(.5));
				this.setZIndex(0);
			};
			marker.addEventListener = marker.addListener;
			return marker;
			function Icon(scale) {
				return {
					url: "img/lbs/marker@2x.png",
					size: new google.maps.Size(30 * scale, 42 * scale),
					anchor: new google.maps.Point(15 * scale, 42 * scale),
					scaledSize: new google.maps.Size(30 * scale, 42 * scale)
				};
			}
		},
		InfoWindow: function (content) {
			return new google.maps.InfoWindow({ content: content });
		},
		Location: function (latlng) { return new google.maps.LatLng(latlng.lat, latlng.lng); },
		Circle: google.maps.Circle,
		YbbMarker: YbbMarker,
		PositionIndicator: PositionIndicator
	};
};
maps.BMap = function () {
	BMap.Map.prototype.panToLocations = function (latlngs) {
		this.setViewport(
			latlngs.map(function (latlng) {
				return new BMap.Point(latlng.lng, latlng.lat);
			})
		);
	};
	function YbbMarker(poi) {
		this.poi = poi;
		var bd = eviltransform.wgs2bd(poi.location.lat, poi.location.lon);
		poi.point = new BMap.Point(bd.lng, bd.lat);
	}
	YbbMarker.prototype = new BMap.Overlay();
	YbbMarker.prototype.initialize = function (map) {
		var element = new Element(this.poi);
		map.getPanes().labelPane.appendChild(element.div);
		this.element = element;
		this.map = map;
		return element.div;
	}
	YbbMarker.prototype.draw = function () {
		var pixel = this.map.pointToOverlayPixel(this.poi.point);
		var div = this.element.div;
		div.style.left = pixel.x + "px";
		div.style.top = pixel.y + "px";
	}
	YbbMarker.prototype.addEventListener = function (event, f) {
		this.element.div['on' + event] = f;
	}
	YbbMarker.prototype.setBadge = function (value) {
		this.element.setBadge(value);
	}
	YbbMarker.prototype.setLabel = function (value) {
		this.element.setLabel(value);
	}
	function PositionIndicator(position) {
		this.position = position;
	}
	PositionIndicator.prototype = new BMap.Overlay();
	PositionIndicator.prototype.initialize = function (map) {
		var element = new maps.PositionIndicator.Element(this.position);

		map.getPanes().labelPane.appendChild(element.img);
		map.addOverlay(element.circle);

		this.element = element;
		this.map = map;

		return element.img;
	}
	PositionIndicator.prototype.draw = function () {
		var bd = eviltransform.wgs2bd(this.position.latitude, this.position.longitude);
		var pixel = this.map.pointToOverlayPixel(bd);
		var img = this.element.img;
		img.style.left = pixel.x + "px";
		img.style.top = pixel.y + "px";
	}
	PositionIndicator.prototype.addEventListener = function (event, f) {
		this.element.img['on' + event] = f;
	}
	PositionIndicator.prototype.setHeading = function (rotation) {
		this.element.setHeading(rotation);
	}
	PositionIndicator.prototype.setPosition = function (position) {
		this.position = position;
		this.draw();
		var circle = this.element.circle;
		var bd = eviltransform.wgs2bd(position.latitude, position.longitude);
		circle.setCenter(bd);
		//accuracy may be null. default to 2000, which is the maximum value returned from API observed
		circle.setRadius(position.accuracy || 2000);
		//workaround circle redrawing bug
		this.map.removeOverlay(circle);
		this.map.addOverlay(circle);
	}
	return {
		Map: function (element) {
			var map = new BMap.Map(element);
			map.setMapStyle({
				styleJson: [{
					featureType: "poi",
					elementType: "all",
					stylers: {
						visibility: "off"
					}
				}, {
					featureType: "building",
					elementType: "all",
					stylers: {
						"visibility": "off"
					}
				}]
			});
			return map;
		},
		Marker: function (options) {
			var marker = new BMap.Marker(options.position, {
				icon: Icon(.5),
				zIndex: 0,
				map: options.map
			});
			marker.activate = function () {
				this.setIcon(Icon(2 / 3));
				this.setZIndex(1);
			};
			marker.deactivate = function () {
				this.setIcon(Icon(.5));
				this.setZIndex(0);
			};
			return marker;
			function Icon(scale) {
				return new BMap.Icon(
					"img/lbs/marker@2x.png",
					new BMap.Size(30 * scale, 42 * scale), {
						anchor: new BMap.Size(15 * scale, 42 * scale),
						imageSize: new BMap.Size(30 * scale, 42 * scale)
					}
				);
			}
		},
		InfoWindow: BMap.InfoWindow,
		Location: function (latlng) { return new BMap.Point(latlng.lng, latlng.lat); },
		Circle: function (options) {
			return new BMap.Circle(
				options.center,
				options.radius,
				options
			);
		},
		YbbMarker: YbbMarker,
		PositionIndicator: PositionIndicator
	};
};
maps.normalize = function (latlng) {
	if (latlng.lat.constructor == Function)
		return {
			lat: latlng.lat(),
			lng: latlng.lng()
		};
	return latlng;
};
maps.Element = Element;
function Element(poi) {
	var img = document.createElement("img");

	var bg = document.createElement("div");
	bg.setAttribute('name', 'bg');

	var badge = document.createElement('div');
	badge.setAttribute('name', 'badge');

	var label = document.createElement('div');
	label.setAttribute('name', 'label');
	label.innerHTML = poi.topic ? poi.topic + '：' + poi.name : poi.name;
	var div = document.createElement('div');
	div.className = 'ybb-marker';

	if (poi.topic)
		$(div).addClass('qualified');
	img.src = environment.imagePrefix + window.categories[poi.type].icon;

	bg.appendChild(img);
	bg.appendChild(badge);
	div.appendChild(bg);
	div.appendChild(label);

	this.img = img;
	this.badge = badge;
	this.label = label;
	this.div = div;
}
Element.prototype.setBadge = function (value) {
	var badge = this.badge;
	if (value) {
		badge.innerHTML = value > 99 ? "99<sup>+</sup>" : value;
		badge.style.display = 'flex';
		badge.style.fontSize = value > 99 ? "80%" : "";
	} else
		badge.style.display = 'none';
}
Element.prototype.setLabel = function (value) {
	var label = this.label;
	if (value) {
		label.innerHTML = value;
		label.style.display = 'flex';
	} else
		label.style.display = 'none';
}
maps.PositionIndicator = {};
maps.PositionIndicator.Element = function (position) {
	var img = document.createElement("img");
	with (img) {
		className = 'position-indicator';
		src = "img/lbs/position-indicator.png";
		style.zIndex = 1;
	}

	var gcj = eviltransform["wgs2" + (maps.m == maps.gmap ? 'gcj' : 'bd')](position.latitude, position.longitude);
	var circle = new maps.m.Circle({
		strokeColor: 'transparent',
		fillColor: 'blue',
		fillOpacity: .2,
		center: maps.m.Location({ lat: gcj.lat, lng: gcj.lng }),
		radius: position.accuracy
	});

	this.img = img;
	this.circle = circle;
};
maps.PositionIndicator.Element.prototype.setHeading = function (rotation) {
	this.img.style.transform = "translate(-50%,-68.75%) rotate(" + rotation + 'deg' + ')';
	this.img.style.webkitTransform = "translate(-50%,-68.75%) rotate(" + rotation + 'deg' + ')';
};
