angular.module('starter.services')
	.service('$db', function ($q) {
		this.open = function (name, onupgradeneeded) {
			var d = $q.defer();
			var request = indexedDB.open(name, 2);
			request.onerror = function (e) { d.reject(e); };
			request.onsuccess = function (e) { d.resolve(e.target.result); };
			request.onupgradeneeded = onupgradeneeded;
			return d.promise;
		};
		this.delete = function (name) {
			var d = $q.defer();
			var req = indexedDB.deleteDatabase(name);
			req.onsuccess = function () { d.resolve(); };
			req.onerror = function (e) { d.reject(); };
			req.onblocked = function () { d.reject('blocked'); };
			return d.promise;
		};
		this.get = function (db, name, id) {
			var d = $q.defer();
			var tx = db.transaction([name], 'readonly');
			var request = tx.objectStore(name).get(id);
			tx.oncomplete = function () {
				d.resolve(request.result);
			};
			tx.onerror = function () {
				d.reject(request.error);
			};
			return d.promise;
		};
		this.set = function (db, name, id, data) {
			var d = $q.defer();
			var tx = db.transaction([name], 'readwrite');
			var request = tx.objectStore(name).put(data, id);
			request.onsuccess = function () { d.resolve(); };
			request.onerror = function () { d.reject(); };
			return d.promise;
		};
		this.remove = function (db, name, id) {
			var d = $q.defer();
			var tx = db.transaction([name], 'readwrite');
			var request = tx.objectStore(name).delete(id);
			request.onsuccess = function () { d.resolve(); };
			request.onerror = function () { d.reject(); };
			return d.promise;
		};
		this.getAll = function (db, name) {
			var d = $q.defer();
			var tx = db.transaction([name], 'readonly');
			var request = tx.objectStore(name).getAll();
			request.onsuccess = function (e) {
				d.resolve(e.target.result);
			};
			request.onerror = function (e) {
				d.reject(e.target.error);
			};
			return d.promise;
		};
		this.clear = function (db, name) {
			var d = $q.defer();
			var tx = db.transaction([name], 'readwrite');
			var request = tx.objectStore(name).clear();
			tx.oncomplete = function () {
				d.resolve();
			};
			tx.onerror = function () {
				d.reject(request.error);
			};
			return d.promise;
		};
		this.foreach = function (db, name, f) {
			var transaction = db.transaction([name], "readonly");
			var objectStore = transaction.objectStore(name);
			objectStore.openCursor().onsuccess = function (event) {
				var cursor = event.target.result;
				if (cursor) {
					f(cursor.value, cursor.key);
					cursor.continue();
				}
			};
		};
	});
