angular.module('starter.services')
.service('$request', function ($storage, $http, $q, $resource) {
	this.get = function (type, id) {
		return $storage.get(type, id)
			.then(function (object) {
				var now = new Date().valueOf();
				if (object && now - object.$time < $resource[type].expiry)
					return object;
				else {
					if (object)
						var config = {
							headers: {
								"If-Modified-Since": new Date(object.$time).toGMTString()
							}
						};
					return $http.get("/youbanban/" + type + '/' + id, config)
						.then(function (response) {
							var data = response.data;
							data.$time = now;
							$storage.set(type, id, data);
							return data;
						}, function (response) {
							if (response.status == 304) {
								object.$time = now;
								$storage.set(type, id, object);
								return object;
							} else
								return $q.reject(response);
						});
				}
			})
	};
	this.get.s = function (type, ids) {
		var $return = {};
		for (var i in ids) {
			var id = ids[i];
			if (!(id in $return))
				$return[id] = $storage.get(type, id);
		}
		var ids = Object.keys($return);

		return $q.all($return)
			.then(function (objects) {
				var uncached = [], expired = [];
				for (var id in objects) {
					var object = objects[id];
					if (object) {
						$return[id] = object;
						if (now - object.$time > $resource[type].expiry)
							expired.push(id);
					} else
						uncached.push(id);
				}
				console.debug(
					ids.length - uncached.length + '/' +
					ids.length +
					" from cache"
				)
				if (!uncached.length)
					return $return;
				var now = new Date().valueOf();
				var query =
					uncached.map(function (id) {
						return {
							id: id,
							since: 0
						};
					}).concat(
						expired.map(function (id) {
							return {
								id: id,
								since: objects[id].$time
							};
						}));
				var batchsize = 400;
				return (query.length <= batchsize ? get(type, query) : batch(type, query)).then(function (response) {
					for (var id in response.data) {
						var object = response.data[id];
						if (object)
							$return[id] = object;
						$return[id].$time = now;
						$storage.set(type, id, $return[id]);
					}
					return $return;
				});
				function slice(array) {
					var a = [];
					for (var i = 0; i < Math.ceil(array.length / batchsize) ; i++)
						a.push(array.slice(i * batchsize, i * batchsize + batchsize));
					return a;
				}
				function get(type, query) {
					if (type == 'poi' && query.length > 100)
						return $http.post("/youbanban/" + type + '/', query);
					return $http.get("/youbanban/" + type + '/', {
						params: {
							query: JSON.stringify(query)
						}
					});
				}
				function batch(type, query) {
					return $q.all(
						slice(query).map(function (s) {
							return get(type, s);
						})
					).then(function (response) {
						return {
							data: angular.extend.apply(undefined, response.map(
								function (r) { return r.data; }
							))
						};
					});
				}
			});
	};
	this.get.s.any = function (ids) {
		var $return = {};
		for (var i in ids) {
			var id = ids[i];
			var type = id.substr(0, id.indexOf('.'));
			if (!(id in $return))
				$return[id] = $storage.get(type, id);
		}
		var ids = Object.keys($return);

		return $q.all($return)
			.then(function (objects) {
				var uncached = [], expired = [];
				for (var id in objects) {
					var object = objects[id];
					if (object) {
						$return[id] = object;
						var type = id.substr(0, id.indexOf('.'));
						if (now - object.$time > $resource[type].expiry)
							expired.push(id);
					} else
						uncached.push(id);
				}
				console.debug(
					ids.length - uncached.length + '/' +
					ids.length +
					" from cache"
				)
				if (!uncached.length)
					return $return;
				var now = new Date().valueOf();
				return $http.get("/youbanban/any/", {
					params: {
						query: JSON.stringify(
							uncached.map(function (id) {
								return {
									id: id,
									since: 0
								};
							}).concat(
								expired.map(function (id) {
									return {
										id: id,
										since: objects[id].$time
									};
								}))
						)
					}
				}).then(function (response) {
					for (var id in response.data) {
						var object = response.data[id];
						if (object)
							$return[id] = object;
						$return[id].$time = now;
						var type = id.substr(0, id.indexOf('.'));
						$storage.set(type, id, $return[id]);
					}
					return $return;
				});
			});
	};
});
