angular.module('starter.services')
.service('cityService', function (collectionCache, $q, $geocode) {
	this.geocode = geocode;
	function geocode(latlng) {
		return $geocode(latlng).then(function (results) {
			var result = filterResult(results);
			console.log(results);
			return map(result).then(function (city) {
				return city || normalize(result);
			});
		});
	}
	function filterResult(results) {
		var political = results.filter(function (result) {
			return result.types.includes('political');
		});
		var locality = political.filter(function (result) {
			return result.types.includes('locality');
		});
		if (!locality.length)
			locality = political.filter(function (result) {
				return result.types.includes('administrative_area_level_1');
			});
		return (locality.length ? locality[0] : political[0]).address_components;
	}
	function normalize(components) {
		var locality = components.find(function (component) {
			return component.types.includes('locality');
		});
		var administrative = components.find(function (component) {
			return component.types.includes('administrative_area_level_1');
		});
		var country = components.find(function (component) {
			return component.types.includes('country');
		});
		var city = locality ? locality.long_name : undefined;
		var province = administrative ? administrative.long_name : undefined;
		console.log([country.long_name, province, city].join('-'));
		return {
			country: country.long_name,
			province: province,
			city: city
		};
	}
	function map(address) {
		return collectionCache.citymap()
			.then(function (citymap) {
				var country = getCountry(address);
				var map = citymap[country];
				var matches = map.map(function (map) {
					return { map: map, m: match(map.original, address) };
				}).filter(function (m) { return m.m != undefined; });
				if (!matches.length) return;
				matches.sort(function (m, n) {
					return n.m - m.m;
				});
				matches = matches.map(function (m) { return m.map; });
				console.log(serialize(matches[0]));
				return {
					city: matches[0].city,
					province: matches[0].province,
					country: matches[0].country
				};
			})
		function serialize(special_area) {
			var original = special_area.original;
			return (
				special_area.original.country + '-' + original.province.join('-') + '-' + original.city
				+
				' -> '
				+
				[special_area.country, special_area.province, special_area.city].join('-')
			);
		}
		function getCountry(address) {
			return address.find(function (component) {
				return (
					component.types.includes('political')
					&&
					component.types.includes('country')
				);
			}).long_name;
		}
		function match(original, address) {
			address = normalize(address);
			var m = 0;
			var language = isCh(
				address.province.find(function (p) { return p; })
				||
				address.city
				||
				''
			) ? 'ch' : 'en';
			var province = language == 'ch' ? original.province : original.provinceEn;
			for (var i in province)
				if (province[i] != '*')
					if (province[i] != address.province[i])
						return;
					else
						m++;
			if (original.city != '*')
				if (original.city != address.city)
					return;
				else
					m++;
			//workaround all-* map
			if (province.filter(function (p) { return p == '*'; }).length == 5 && original.city == '*')
				return;
			return m;
		}
		function normalize(address) {
			var $ret = {
				province: [null, null, null, null, null],
				city: null
			};
			var locality = null;
			var sublocality = [null, null, null, null, null];
			for (var i in address) {
				var component = address[i];
				var type = component.types.find(function (type) { return type != 'political'; });
				if (type.includes("admin"))
					$ret.province[type.substr(-1) - 1] = component.long_name;
				if (type.includes("locality"))
					locality = component.long_name;
				if (type.includes("sublocality"))
					sublocality[type.substr(-1) - 1] = component.long_name;
			}
			$ret.city = locality || sublocality.find(function (x) { return x; });
			return $ret;
		}
		function isCh(s) {
			return s.match(/[\u3400-\u9FBF]/);
		}
	}
})