angular.module('starter.services')
	.service('collectionCache', function ($http, $db, $q) {
		var collections = {
			city: {
				url: "/youbanban/config/areas",
				expiry: 6*60*60*1000,
				transform: function (data) { return data; }
			},
			hotCity: {
				url: "/youbanban/config/hot_areas",
				expiry: 6*60*60*1000,
				transform: function (data) { return data; }
			},
			homepageCity: {
				url: "/youbanban/homepage/areas",
				expiry: 6*60*60*1000,
				transform: function (data) { return data; }
			},
			citymap: {
				url: "/youbanban/config/city_map",
				expiry: 6*60*60*1000,
				transform: function (data) {
					data = data.groupBy(function (map) { return map.original.country; });
					return data;
				}
			},
			topics: {
				url: "/youbanban/config/topic",
				expiry: 6 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			},
			offline: {
				url: "/youbanban/config/offline",
				expiry: 30000,
				transform: function (data) { return data; }
			},
			category: {
				url: "/youbanban/config/category",
				expiry: 24 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			},
			"user.ybb": {
				url: "/youbanban/user/user.ybb",
				expiry: 24 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			},
			language: {
				url: "/youbanban/batman/translation/languages",
				expiry: 24 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			},
			areaCode: {
				url: "/youbanban/config/dialing_area",
				expiry: 24 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			},
			usualAreaCode: {
				url: "/youbanban/config/dialing_area/common",
				expiry: 24 * 60 * 60 * 1000,
				transform: function (data) { return data; }
			}
		};
		var db;
		var promise = $db.open('collection', function (e) {
			var db = e.target.result;
			var stores = ['.', 'time'];
			for (var i in stores)
				if (!db.objectStoreNames.contains(stores[i]))
					db.createObjectStore(stores[i]);
		}).then(function (_db) {
			db = _db;
		});
		var $this = {};
		angular.forEach(collections, function (collection, i) {
			$this[i] = function () {
				return $db.get(db, 'time', i)
					.then(function (time) {
						var now = new Date();
						if (!time || now - time > collection.expiry) {
							if (time)
								var config = {
									headers: {
										"If-Modified-Since": new Date(time).valueOf()
									}
								};
							return $http.get(collection.url, config)
								.then(function (response) {
									var data = collection.transform(response.data);
									$db.set(db, '.', i, data);
									$db.set(db, 'time', i, now.valueOf())
									return data;
								}, function (response) {
									if (response.status == 304) {
										$db.set(db, 'time', i, now.valueOf())
										return $db.get(db, '.', i);
									} else
										return $q.reject(response);
								});
						} else
							return $db.get(db, '.', i);
					})
			};
		});
		angular.forEach(collections, function (collection, i) {
			var f = $this[i];
			$this[i] = function () {
				return db ?
					f() :
					promise.then(f);
			};
		});
		return $this;
	});
