angular.module('starter.services')
.service('moodService', function ($timeout, photoService, $q, $http, $rest, $storage, $file, $ionicLoading, $rootScope) {
	this.getDraft = function () {
		return $http.get('/youbanban/user/' + $rootScope.user + '/mood_draft')
			.then($rest.join.any());
	};
	this.post = function (mood, option) {
		return post(mood, option);
	};
	this.copy = copy;
	this.equals = equals;
	function copy(mood) {
		return {
			id: mood.id,
			rich: mood.rich,
			title: mood.title,
			richContent: angular.copy(mood.richContent)
		};
	}
	function equals(a, b) {
		return a.title == b.title && angular.equals(a.richContent, b.richContent);
	}
	function compressImages(values) {
		var exifStrs = [];
		var compressUrl = [];
		return $q.all(
			//通过url获取图片dataUrl
			values.map(photoService.getDataUrl)
		).then(function (dataUrl) {
			//提取exif信息保存到exifStrs
			exifStrs = dataUrl.map(function (v) {
				var exifObj = piexif.load(v);
				return piexif.dump(exifObj);
			});
			//压缩图片
			return photoService.compress(values)
		}).then(function (values) {
			//将压缩图片url保存下来
			compressUrl = values;
			//获取压缩图片的dataUrl
			return $q.all(values.map(photoService.getDataUrl));
		}).then(function (dataUrl) {
			return $q.all(dataUrl.map(function (v, i) {
					//写入exif
					var data = piexif.insert(exifStrs[i], v);
					return photoService.writeImg(data, compressUrl[i]);
				})
			)
		}).then(function () {
			return compressUrl;
		});
	}
	function post(mood, option) {
		var md5, metadata;
		var components = mood.richContent.filter(function (component) {
			return component.type == 'picture';
		}).filter(function (component) {
			return component.value.indexOf('file://') == 0;
		});
		var values = components.map(function (component) {
			return component.value;
		});
		return (option.useOriginalImage ?
			$q.resolve(values) :
			compressImages(values)
		).then(function (images) {
			return $q.all({
				md5: $q.all(images.map(photoService.getMd5)),
				metadata: $q.all(images.map($file.getMetadata))
			}).then(function (_) {
				md5 = _.md5,
				metadata = _.metadata;
				return md5.map(function (md5, i) {
					return {
						id: i + 1,
						checksum: md5,
						timestamp: new Date(metadata[i].modificationTime).valueOf(),
						size: metadata[i].size
					};
				});
			}).then(function (images) {
				$ionicLoading.show({ template: "<ion-spinner></ion-spinner><div>正在准备上传照片</div>" });
				return $http.get("/ybbimg/check_images", {
					params: {
						query: JSON.stringify(images)
					}
				});
			}).then(function (response) {
				var pictures = response.data.map(function (picture) {
					return picture.path ? picture.path : '';
				});
				var files = response.data.filter(function (picture) {
					return !picture.path;
				}).map(function (picture) {
					return picture.id - 1;
				});
				var checksums = files.map(function (i) {
					return md5[i];
				});
				return form(files.map(function (i) {
					return components[i].value;
				})).then(function (data) {
					for (var i in components)
						components[i].value = pictures[i];
					$ionicLoading.show({ template: "<ion-spinner></ion-spinner><progress-bar></progress-bar><div name=\"progress\"></div>" })
					return $http[mood.id ? 'put' : 'post']("/youbanban/mood/" + (mood.id || ""), data, {
						transformRequest: angular.identity,
						params: {
							checksums: JSON.stringify(checksums),
							data: mood,
							files: ['files']
						},
						headers: {
							'Content-Type': undefined,
							__PROGRESS__: function () {
								return function (xhr) {
									xhr.upload.addEventListener("progress", function (event) {
										console.log(event.loaded + '/' + event.total);
										var progress = event.loaded / event.total;
										var percent = Math.floor(progress * 100);
										$(".loading-container .progress-bar>div").width(percent + "%");
										$(".loading-container div[name=\"progress\"]").text(percent + "%");
									});
									xhr.upload.addEventListener("load", function (event) {
										console.log("upload complete");
										$ionicLoading.show({ template: "<ion-spinner></ion-spinner><div>正在创建足迹</div>" })
									});
								};
							}
						},
						timeout: 60 * 60 * 1000
					});
				}).then(function (response) {
					var method = mood.id ? 'put' : 'post';
					var id = mood.id || response.data;
					return $http({
						url: '/youbanban/mood/' + id + '/privacy',
						method: 'PUT',
						data:option.private ? 0 : 1
					}).then(function () {
					return $http.get("/youbanban/mood/" + id)
						.then(function (response) {
							mood.richContent = angular.copy(response.data.richContent);
							mood.id = response.data.id;
							$storage.set('mood', id, response.data);
							var event = { post: 'create', put: 'modify' }[method];
							$rootScope.$broadcast('mood.' + event, response.data);
							return response;
						});
					});
				});
				function form(pictures) {
					return $q.all(pictures.map($file.readAsArrayBuffer))
						.then(function (buffers) {
							var data = new FormData();
							var sum = 0;
							for (var i in buffers) {
								data.append('files', new Blob([buffers[i]], { type: "image/jpeg" }), "files" + i + ".jpg");
								sum += buffers[i].byteLength;
							}
							return data;
						});
				}
			});
		})
	}
});
